# Generated by Django 3.1 on 2022-09-15 04:48

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('telegram_django_bot', '0003_botmenuelem_message_format'),
    ]

    operations = [
        migrations.AlterField(
            model_name='botmenuelem',
            name='buttons_db',
            field=models.TextField(blank=True, help_text='InlineKeyboardMarkup buttons list, ({"text": , "url" or "callback_data"})', null=True),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='callbacks_db',
            field=models.TextField(blank=True, help_text='List of regular expressions (so far only an explicit list) for callbacks that call this menu block', null=True),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='command',
            field=models.CharField(blank=True, help_text='Bot command that can call this menu block', max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='empty_block',
            field=models.BooleanField(default=False, help_text='This block will be shown if there is no catching callback'),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='is_visable',
            field=models.BooleanField(default=True, help_text='Whether to display this menu block to users (can be hidden and not deleted for convenience)'),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='media',
            field=models.FileField(blank=True, help_text='File attachment to the message', null=True, upload_to=''),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='message',
            field=models.TextField(help_text='Text message'),
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='telegram_file_code',
            field=models.CharField(blank=True, help_text='File code in telegram (must be deleted when replacing file)', max_length=512, null=True),
        ),
        migrations.AlterField(
            model_name='teledeeplink',
            name='link',
            field=models.CharField(max_length=64, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9_-]+$', 'Telegram is only accepted letters, numbers and signs - _')]),
        ),
        migrations.AlterField(
            model_name='trigger',
            name='botmenuelem',
            field=models.ForeignKey(help_text='which trigger message to show', on_delete=django.db.models.deletion.PROTECT, to='telegram_django_bot.botmenuelem'),
        ),
        migrations.AlterField(
            model_name='trigger',
            name='min_duration',
            field=models.DurationField(help_text='the minimum period in which there can be 1 notification for a user of this type'),
        ),
        migrations.AlterField(
            model_name='trigger',
            name='priority',
            field=models.IntegerField(default=1, help_text='the more topics will be executed first'),
        ),
        migrations.CreateModel(
            name='BotMenuElemAttrText',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dttm_added', models.DateTimeField(default=django.utils.timezone.now)),
                ('language_code', models.CharField(max_length=2)),
                ('default_text', models.TextField(help_text='The text which should be translate')),
                ('translated_text', models.TextField(help_text='Default_text Translation', null=True)),
                ('bot_menu_elem', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='telegram_django_bot.botmenuelem')),
            ],
            options={
                'unique_together': {('bot_menu_elem', 'language_code', 'default_text')},
                'index_together': {('bot_menu_elem', 'language_code', 'default_text')},
            },
        ),
        migrations.AlterField(
            model_name='botmenuelem',
            name='message_format',
            field=models.CharField(
                choices=[('T', 'Text'), ('P', 'Image'), ('D', 'Document'), ('A', 'Audio'), ('V', 'Video'),
                         ('G', 'GIF/animation'), ('GM', 'Media Group')], default='T', max_length=2),
        ),
    ]



import logging
from .models import BotMenuElem
from .utils import handler_decor
from .td_viewset import TelegaViewSet
from django.urls import resolve, Resolver404, reverse
from django.conf import settings
from django.contrib.auth import get_user_model

import telegram
import inspect

from telegram.ext import Handler, CallbackQueryHandler, Filters


def telega_resolve(path, utrl_conf=None):
    if path[0] != '/':
        path = f'/{path}'

    if '?' in path:
        path = path.split('?')[0]

    if utrl_conf is None:
        utrl_conf = settings.TELEGRAM_ROOT_UTRLCONF

    try:
        resolver_match = resolve(path, utrl_conf)
    except Resolver404:
        resolver_match = None
    return resolver_match


def telega_reverse(viewname, utrl_conf=None, args=None, kwargs=None, current_app=None):
    if utrl_conf is None:
        utrl_conf = settings.TELEGRAM_ROOT_UTRLCONF

    response = reverse(viewname, utrl_conf, args, kwargs, current_app)
    if response[0] == '/':
        response = response[1:]
    return response


@handler_decor(log_type='C')
def all_command_bme_handler(bot, update, user):

    if len(update.message.text[1:]) and 'start' == update.message.text[1:].split()[0]:
        menu_elem = None
        if len(update.message.text[1:]) > 6:  # 'start ' + something
            menu_elem = BotMenuElem.objects.filter(
                command__contains=update.message.text[1:],
                is_visable=True,
            ).first()

        if menu_elem is None:
            menu_elem = BotMenuElem.objects.filter(
                command='start',
                is_visable=True,
            ).first()
    else:
        menu_elem = BotMenuElem.objects.filter(
            command=update.message.text[1:],
            is_visable=True,
        ).first()
    return bot.send_botmenuelem(update, user, menu_elem)


@handler_decor(log_type='C')
def all_callback_bme_handler(bot, update, user):
    menu_elem = BotMenuElem.objects.filter(
        callbacks_db__contains=update.callback_query.data,
        is_visable=True,
    ).first()
    return bot.send_botmenuelem(update, user, menu_elem)



class RouterCallbackMessageCommandHandler(Handler):
    def __init__(self, utrl_conf=None, only_utrl=False, **kwargs):
        kwargs['callback'] = lambda x: 'for base class'
        super().__init__(**kwargs)
        self.callback = None
        self.utrl_conf = utrl_conf
        self.only_utrl = only_utrl # without BME elems

    def get_callback_utrl(self, update):
        callback_func = None
        # check if utrls
        if update.callback_query:
            callback_func = telega_resolve(update.callback_query.data, self.utrl_conf)
        elif update.message and update.message.text and update.message.text[0] == '/':  # is it ok? seems message couldnt be an url
            callback_func = telega_resolve(update.message.text, self.utrl_conf)

        if callback_func is None:
            # update.message -- could be data or info for managing, command could not be a data, it is managing info
            if update.message and (update.message.text is None or update.message.text[0] != '/'):
                user_details = update.message.from_user

                user = get_user_model().objects.filter(id=user_details.id).first()
                if user:
                    logging.info('user.current_utrl', user.current_utrl)
                    if user.current_utrl:
                        callback_func = telega_resolve(user.current_utrl, self.utrl_conf)
        return callback_func

    def check_update(self, update: object):
        """
        check if callback or message (command actually is message)
        :param update:
        :return:
        """
        if isinstance(update, telegram.Update) and (update.effective_message or update.callback_query):
            callback = self.get_callback_utrl(update)
            if callback:
                return True
            elif not self.only_utrl:
                if update.message and update.message.text and update.message.text[0] == '/':
                    # if it is a command then it should be  early in handlers
                    # or in BME (then return True
                    return True
                elif update.callback_query:
                    return True
        return None

    def handle_update(
        self,
        update,
        dispatcher,
        check_result: object,
        context=None,
    ):
        # todo: add flush utrl and data if viewset utrl change or error

        callback_func = self.get_callback_utrl(update)

        if not callback_func is None:
            if inspect.isclass(callback_func.func) and issubclass(callback_func.func, TelegaViewSet):
                viewset = callback_func.func(callback_func.route)

                decorating = handler_decor(log_type='N',)

                callback_func = decorating(viewset.dispatch)

            else:
                callback_func = callback_func.func

        # check if in BME (we do not need check only_utrl here, as there was a check in self.check_update)
        if callback_func is None:
            if update.callback_query:
                callback_func = all_callback_bme_handler
            else:
                callback_func = all_command_bme_handler

        self.collect_additional_context(context, update, dispatcher, check_result)
        return callback_func(update, context)





# Generated by Django 3.1 on 2022-07-12 03:00

import datetime
from django.conf import settings
import django.contrib.auth.models
import django.contrib.auth.validators
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import telegram_django_bot.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('seed_code', models.IntegerField(default=telegram_django_bot.models._seed_code)),
                ('telegram_username', models.CharField(blank=True, max_length=64, null=True)),
                ('timezone', models.DurationField(default=datetime.timedelta(seconds=10800))),
                ('message_write_code', models.CharField(choices=[('D', 'default'), ('TV', 'target value'), ('TN', 'target name'), ('PV', 'payment value'), ('RV', 'report value'), ('DN', 'WRITE_CODE_DAY_REPORT_NAME')], default='D', max_length=3)),
                ('message_write_code_dttm', models.DateTimeField(blank=True, null=True)),
                ('message_write_code_context', models.CharField(blank=True, default='', max_length=128)),
                ('tax', models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True)),
                ('activated', models.BooleanField(default=False)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='BotMenuElem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('command', models.CharField(blank=True, help_text='Команда бота, по которой можно вызвать этот блок меню', max_length=32, null=True, unique=True)),
                ('empty_block', models.BooleanField(default=False, help_text='если забыли и нет события ловяшего коллбек, то это покажет')),
                ('is_visable', models.BooleanField(default=True, help_text='Отображать ли этот блок меню пользователям (можно скрыть и не удалять для удобства)')),
                ('callbacks_db', models.TextField(blank=True, help_text='список регулярных выражений (пока только явный список) для коллбеков, которые вызывают это блок меню', null=True)),
                ('forward_message_id', models.IntegerField(blank=True, null=True)),
                ('forward_chat_id', models.IntegerField(blank=True, null=True)),
                ('message', models.TextField(help_text='Текстовое сообщение')),
                ('buttons_db', models.TextField(blank=True, help_text='InlineKeyboardMarkup список кнопок, ({"text": , "url" or "callback_data"})', null=True)),
                ('media', models.FileField(blank=True, help_text='Файл приложение к сообщению', null=True, upload_to='')),
                ('telegram_file_code', models.CharField(blank=True, help_text='Код файла в телеграмме (удалить при замене файла)', max_length=512, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Trigger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dttm_added', models.DateTimeField(default=django.utils.timezone.now)),
                ('dttm_deleted', models.DateTimeField(blank=True, null=True)),
                ('name', models.CharField(max_length=512, unique=True)),
                ('condition_db', models.TextField(help_text="\n    {\n        seeds: [1, 2, 3, 4, 5],\n        'amount': [{\n            'gte': 5,\n            'type__contains': 'dd',  // type__in, type\n            'duration': '7d'\n        }]\n    }\n    ")),
                ('min_duration', models.DurationField(help_text='минимальный период, в который может быть 1 уведомление для пользователя данного типа')),
                ('priority', models.IntegerField(default=1, help_text='чем больше тем в первую очередь будут выполняться')),
                ('botmenuelem', models.ForeignKey(help_text='какое сообщение по тригеру показываем', on_delete=django.db.models.deletion.PROTECT, to='telegram_django_bot.botmenuelem')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserTrigger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dttm_added', models.DateTimeField(default=django.utils.timezone.now)),
                ('dttm_deleted', models.DateTimeField(blank=True, null=True)),
                ('message_id', models.BigIntegerField(blank=True, null=True)),
                ('is_sent', models.BooleanField(default=False)),
                ('trigger', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='telegram_django_bot.trigger')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TeleDeepLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, default='', max_length=64)),
                ('price', models.FloatField(blank=True, null=True)),
                ('link', models.CharField(max_length=64, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9_-]+$', 'Телеграм принимает только буквы, цифры и знаки - _ ')])),
                ('users', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ActionLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dttm', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('type', models.CharField(max_length=32)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
